@@ -2,8 +2,6 @@
 #define _NFTABLES_IFACE_H_
 
 #include <net/if.h>
-#include <list.h>
-#include <stdint.h>
 
 struct iface {
 	struct list_head	list;
@@ -17,5 +15,4 @@ char *nft_if_indextoname(unsigned int ifindex, char *name);
 void iface_cache_update(void);
 void iface_cache_release(void);
 
-const struct iface *iface_cache_get_next_entry(const struct iface *prev);
 #endif
