@@ -11,8 +11,8 @@
  * programme.
  */
 
-#define _GNU_SOURCE
-#include <string.h>
+#include <nft.h>
+
 #include <errno.h>
 #include <inttypes.h>
 #include <nftables.h>
@@ -396,6 +396,7 @@ static int rule_collect_stmts(struct optimize_ctx *ctx, struct rule *rule)
 				clone->ops = &unsupported_stmt_ops;
 				break;
 			}
+			/* fall-through */
 		case STMT_VERDICT:
 			clone->expr = expr_get(stmt->expr);
 			break;
